<?php

class TestMapMetaCap extends WPTestCase {
	function setUp() {
		parent::setUp();

		$this->post_type = rand_str( 20 );
		register_post_type( $this->post_type );

		$this->user_id = 1;
		$this->author_id = $this->user_id + 1;

		$this->post_id = wp_insert_post( array(
			'post_title' => rand_str(),
			'post_type' => $this->post_type,
			'post_status' => 'private',
			'post_author' => $this->author_id,
		) );

	}

	function tearDown() {
		parent::tearDown();
		wp_delete_post( $this->post_id, true );
	}

	function test_capability_type_post_with_no_extra_caps() {

		register_post_type( $this->post_type, array(
			'capability_type' => 'post',
		) );
		$post_type_object = get_post_type_object( $this->post_type );

		$this->assertTrue( $post_type_object->map_meta_cap );

		$this->assertEquals( array( 'edit_others_posts', 'edit_private_posts' ),
			map_meta_cap( 'edit_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'edit_others_posts', 'edit_private_posts' ),
			map_meta_cap( $post_type_object->cap->edit_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'read_private_posts' ),
			map_meta_cap( 'read_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'read_private_posts' ),
			map_meta_cap( $post_type_object->cap->read_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'delete_others_posts', 'delete_private_posts' ),
			map_meta_cap( 'delete_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'delete_others_posts', 'delete_private_posts' ),
			map_meta_cap( $post_type_object->cap->delete_post, $this->user_id, $this->post_id ) );
	}

	function test_custom_capability_type_with_map_meta_cap() {
		register_post_type( $this->post_type, array(
			'capability_type' => 'book',
			'map_meta_cap' => true,
		) );

		$post_type_object = get_post_type_object( $this->post_type );

		$this->assertEquals( array( 'edit_others_books', 'edit_private_books' ),
			map_meta_cap( 'edit_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'edit_others_books', 'edit_private_books' ),
			map_meta_cap( $post_type_object->cap->edit_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'read_private_books' ),
			map_meta_cap( 'read_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'read_private_books' ),
			map_meta_cap( $post_type_object->cap->read_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'delete_others_books', 'delete_private_books' ),
			map_meta_cap( 'delete_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'delete_others_books', 'delete_private_books' ),
			map_meta_cap( $post_type_object->cap->delete_post, $this->user_id, $this->post_id ) );
	}

	function test_capability_type_post_with_one_renamed_cap() {
		register_post_type( $this->post_type, array(
			'capability_type' => 'post',
			'capabilities' => array( 'edit_posts' => 'edit_books' ),
		) );

		$post_type_object = get_post_type_object( $this->post_type );

		$this->assertFalse( $post_type_object->map_meta_cap );

		$this->assertEquals( array( 'edit_post' ),
			map_meta_cap( 'edit_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'edit_post' ),
			map_meta_cap( $post_type_object->cap->edit_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'read_post' ),
			map_meta_cap( 'read_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'read_post' ),
			map_meta_cap( $post_type_object->cap->read_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'delete_post' ),
			map_meta_cap( 'delete_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'delete_post' ),
			map_meta_cap( $post_type_object->cap->delete_post, $this->user_id, $this->post_id ) );
	}

	function test_capability_type_post_map_meta_cap_true_with_renamed_cap() {
		register_post_type( $this->post_type, array(
			'capability_type' => 'post',
			'map_meta_cap' => true,
			'capabilities' => array(
				'edit_post' => 'edit_book', // maps back to itself.
				'edit_others_posts' => 'edit_others_books',
			),
		) );

		$post_type_object = get_post_type_object( $this->post_type );

		$this->assertTrue( $post_type_object->map_meta_cap );

		$this->assertEquals( array( 'edit_others_books', 'edit_private_posts' ),
			map_meta_cap( 'edit_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'edit_others_books', 'edit_private_posts' ),
			map_meta_cap( $post_type_object->cap->edit_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'read_private_posts' ),
			map_meta_cap( 'read_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'read_private_posts' ),
			map_meta_cap( $post_type_object->cap->read_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'delete_others_posts', 'delete_private_posts' ),
			map_meta_cap( 'delete_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'delete_others_posts', 'delete_private_posts' ),
			map_meta_cap( $post_type_object->cap->delete_post, $this->user_id, $this->post_id ) );
	}

	function test_capability_type_post_with_all_meta_caps_renamed() {
		register_post_type( $this->post_type, array(
			'capability_type' => 'post',
			'capabilities' => array(
				'edit_post' => 'edit_book',
				'read_post' => 'read_book',
				'delete_post' => 'delete_book',
			),
		) );

		$post_type_object = get_post_type_object( $this->post_type );

		$this->assertFalse( $post_type_object->map_meta_cap );

		$this->assertEquals( array( 'edit_book' ),
			map_meta_cap( 'edit_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'edit_book' ),
			map_meta_cap( $post_type_object->cap->edit_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'read_book' ),
			map_meta_cap( 'read_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'read_book' ),
			map_meta_cap( $post_type_object->cap->read_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'delete_book' ),
			map_meta_cap( 'delete_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'delete_book' ),
			map_meta_cap( $post_type_object->cap->delete_post, $this->user_id, $this->post_id ) );
	}

	function test_capability_type_post_with_all_meta_caps_renamed_mapped() {
		register_post_type( $this->post_type, array(
			'capability_type' => 'post',
			'map_meta_cap' => true,
			'capabilities' => array(
				'edit_post'   => 'edit_book',
				'read_post'   => 'read_book',
				'delete_post' => 'delete_book',
			),
		) );

		$post_type_object = get_post_type_object( $this->post_type );

		$this->assertTrue( $post_type_object->map_meta_cap );

		$this->assertEquals( array( 'edit_others_posts', 'edit_private_posts' ),
			map_meta_cap( 'edit_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'edit_others_posts', 'edit_private_posts' ),
			map_meta_cap( $post_type_object->cap->edit_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'read_private_posts' ),
			map_meta_cap( 'read_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'read_private_posts' ),
			map_meta_cap( $post_type_object->cap->read_post, $this->user_id, $this->post_id ) );

		$this->assertEquals( array( 'delete_others_posts', 'delete_private_posts' ),
			map_meta_cap( 'delete_post', $this->user_id, $this->post_id ) );
		$this->assertEquals( array( 'delete_others_posts', 'delete_private_posts' ),
			map_meta_cap( $post_type_object->cap->delete_post, $this->user_id, $this->post_id ) );
	}

	function test_file_edit_caps_not_reliant_on_unfiltered_html_constant() {
		$this->knownWPBug(20488);
		define( 'DISALLOW_UNFILTERED_HTML', true );
		if ( defined( 'DISALLOW_FILE_MODS' ) || defined( 'DISALLOW_FILE_EDIT' ) )
			$this->markTestSkipped('DISALLOW_FILE_MODS or DISALLOW_FILE_EDIT is defined.');
		$this->assertEquals( array( 'update_core' ), map_meta_cap( 'update_core', $this->user_id ) );
		$this->assertEquals( array( 'edit_plugins' ), map_meta_cap( 'edit_plugins', $this->user_id ) );
	}
}

?>
